/*****************************************************************************
 *                DigCC: a minimal C compiler for DigMIPS                    *
 ***************************************************************************** 
 *                                                                           *
 * Copyright (c) 2012 Christophe Alias                                       *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License as published by the     *
 * Free Software Foundation; either version 2 of the License, or (at your    * 
 * option) any later version.                                                *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General  *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License along   *
 * with this program; if not, write to the Free Software Foundation, Inc.,   *
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                 *
 *                                                                           *
 * Written by Christophe Alias, Christophe.Alias@ens-lyon.fr                 *
 *                                                                           *
 *****************************************************************************/
#include "BasicBlock.h"

#include <sstream>


BasicBlock::BasicBlock(vector<Code*> vec)
{ 
  // cout << "BasicBlock:" << endl;
  // for(int i=0; i<vec.size(); i++)
  //   cout << "  " << vec[i]->to_string() << endl;
  // cout << endl;

  read_register = new BitVector(0); write_register = new BitVector(0); 
  block = vec; 
}


BitVector* 
BasicBlock::get_read_register()
{
  return read_register;
}


BitVector* 
BasicBlock::get_write_register()
{
  return write_register;
}


string 
BasicBlock::get_label()
{
  return label;
}


int 
BasicBlock::is_jump()
{
  return true; //Always true for a BB
}


int 
BasicBlock::is_label()
{
  return true; //idem
}


BasicBlock* 
BasicBlock::clone()
{
  BasicBlock* result;
  vector<Code*> vec;
  for(int i=0; i<block.size(); i++)
    vec.push_back(block[i]->clone());
  return new BasicBlock(vec);
}


string 
BasicBlock::to_string()
{
  string result;
  stringstream ss;

  for(int i=0; i<block.size(); i++)
    ss << block[i]->to_string() << endl;

  result = ss.str();
  return result;
}


string 
BasicBlock::to_string_dot()
//Mainly a copy/paste of to_string()
{
  string result;
  stringstream ss;

  for(int i=0; i<block.size(); i++)
    ss << block[i]->to_string() << "\\n"; //difference here

  result = ss.str();
  return result;
}
