/*****************************************************************************
 *                DigCC: a minimal C compiler for DigMIPS                    *
 ***************************************************************************** 
 *                                                                           *
 * Copyright (c) 2012 Christophe Alias                                       *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License as published by the     *
 * Free Software Foundation; either version 2 of the License, or (at your    * 
 * option) any later version.                                                *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General  *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License along   *
 * with this program; if not, write to the Free Software Foundation, Inc.,   *
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                 *
 *                                                                           *
 * Written by Christophe Alias, Christophe.Alias@ens-lyon.fr                 *
 *                                                                           *
 *****************************************************************************/
#ifndef __CODE_DIGMIPS_H
#define __CODE_DIGMIPS_H

#include "Code.h" //abstract super-class
#include "PseudoCode.h" //for test_t (cjump relation)
#include "Cfg.h"
#include "Allocation.h"

#include "BitVector.h"

#include <iostream>  //For the pretty-printer
#include <vector>

using namespace std;

enum code_digmips_t
  {
    DIGMIPS_ADD, 
    DIGMIPS_SUB, 
    DIGMIPS_LD, 
    DIGMIPS_ST, 
    DIGMIPS_BLE, 
    DIGMIPS_LDI, 
    DIGMIPS_LDI_LO, 
    DIGMIPS_LDI_HI, 
    DIGMIPS_JA, 
    DIGMIPS_J, 
    DIGMIPS_LABEL,

    DIGMIPS_PUSH_ALL, //
    DIGMIPS_POP_ALL,  // To patched after register allocation

    DIGMIPS_INLINE_ASM
  };

class CodeDigmips: public Code
{
 public:
  code_digmips_t instruction;

  int r_dest;
  int r1,r2;
  int shift;
  int constant;
  string label; //if label
  string asm_code; //convey inline asm

  BitVector* read_register; //use
  BitVector* write_register; //def

  CodeDigmips() 
    {  
      read_register = new BitVector(0); 
      write_register = new BitVector(0);
    };

  //
  // Implementation of abstract methods
  //

  BitVector* get_read_register(); //use
  BitVector* get_write_register(); //def
  string get_label();

  int is_jump();
  int is_label();

  string to_string();
  string to_string_dot();

  vector<Code*> apply(Allocation* a); //spill if required by 'a'

  CodeDigmips* clone();

  int read_r1(); //used for spilling
  int read_r2();
  int read_r_dest();
  int write_r_dest();
};

//Pretty constructors
void digmips_add(Cfg* cfg, int r_dest, int r1, int r2);
void digmips_sub(Cfg* cfg, int r_dest, int r1, int r2);
void digmips_ld(Cfg* cfg, int r_dest, int r_base, int shift);
void digmips_st(Cfg* cfg, int r_dest, int shift, int r_source);
void digmips_ble(Cfg* cfg, int r1, int r2, string label);
void digmips_ldi(Cfg* cfg, int r_dest, int constante);
void digmips_ldi_lo(Cfg* cfg, int r_dest, string label);
void digmips_ldi_hi(Cfg* cfg, int r_dest, string label);
void digmips_ja(Cfg* cfg, int r1, int r2);
void digmips_j(Cfg* cfg, string label);
void digmips_label(Cfg* cfg, string label_id);

void digmips_push_all(Cfg* cfg);
void digmips_pop_all(Cfg* cfg);

void digmips_final_push_all(Cfg* cfg);
void digmips_final_pop_all(Cfg* cfg);

void digmips_inline_asm(Cfg* cfg,string asm_code);

//Macros
void cjump(Cfg* cfg, test_t relation, int r1, int r2, string label);
void copy(Cfg* cfg, int r_dest, int r_source);
void copy_(Cfg* cfg, int r_dest, int r_source);
//use r2 for 0 => no spill, used in the prelude
void push(Cfg* cfg, int r);
void pop(Cfg* cfg, int r);
void push_(Cfg* cfg, int r); 
void pop_(Cfg* cfg, int r);
//use r2 to add/sub 1 => no spill, used in the prelude
void pop(Cfg* cfg, int r);  
void call(Cfg* cfg, string label);
void ret(Cfg* cfg, int n);


/*
 * Main translation functions here.
 */

void digmips_translate(Cfg* cfg, PseudoCode* code);
//Add to the CFG the code translating the given pseudo-code.

void digmips_apply(Cfg* cfg, CodeDigmips* code, Allocation* a);
//Apply the allocation to the code (+spill if required)

string digmips_register(int r);
//Give the actual register id for 'r'

#endif
