/*****************************************************************************
 *                DigCC: a minimal C compiler for DigMIPS                    *
 ***************************************************************************** 
 *                                                                           *
 * Copyright (c) 2012 Christophe Alias                                       *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License as published by the     *
 * Free Software Foundation; either version 2 of the License, or (at your    * 
 * option) any later version.                                                *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General  *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License along   *
 * with this program; if not, write to the Free Software Foundation, Inc.,   *
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                 *
 *                                                                           *
 * Written by Christophe Alias, Christophe.Alias@ens-lyon.fr                 *
 *                                                                           *
 *****************************************************************************/
#include "InterferenceGraph.h"

#include "Register.h"

#include <stdlib.h> //for exit() and system()
#include <fstream> //for show_dot() file I/O

#define SPILL0 0
#define SPILL1 1
#define SP 6
#define ARP 7

InterferenceGraph::InterferenceGraph(Cfg* cfg)
{
  succ.resize(nb_registers());
  for(int i=0; i<nb_registers(); i++)
    succ[i] = new BitVector(nb_registers());

  /*
   * Make interfere each couple of temps belong to a live-in set
   */

  for(int i=0; i<cfg->live_in.size(); i++)
    {
      vector<int> live_in_vec = cfg->live_in[i]->to_vector();
      for(int j=0; j<live_in_vec.size(); j++)
	for(int k=0; k<live_in_vec.size(); k++)
	  if(j != k)
	    //temps live_in_vec[j] and live_in_vec[k] interfere
	    {
	      int temp1 = live_in_vec[j];
	      int temp2 = live_in_vec[k];
	      if((temp1 != ARP) && (temp1 != SP) &&
		 (temp1 != SPILL0) && (temp1 != SPILL1) &&
		 (temp2 != ARP) && (temp2 != SP) &&
		 (temp2 != SPILL0) && (temp2 != SPILL1))
		{
		  succ[temp1]->insert(temp2); //arc temp1 -> temp2
		  succ[temp2]->insert(temp1); //arc temp2 -> temp1
		}
	    }
    }


  /*
   * Same for live-out.
   */

  for(int i=0; i<cfg->live_out.size(); i++)
    {
      vector<int> live_out_vec = cfg->live_out[i]->to_vector();
      for(int j=0; j<live_out_vec.size(); j++)
	for(int k=0; k<live_out_vec.size(); k++)
	  if(j != k)
	    //temps live_out_vec[j] and live_out_vec[k] interfere
	    {
	      int temp1 = live_out_vec[j];
	      int temp2 = live_out_vec[k];
	      if((temp1 != ARP) && (temp1 != SP) &&
		 (temp1 != SPILL0) && (temp1 != SPILL1) &&
		 (temp2 != ARP) && (temp2 != SP) &&
		 (temp2 != SPILL0) && (temp2 != SPILL1))
		{
		  succ[temp1]->insert(temp2); //arc temp1 -> temp2
		  succ[temp2]->insert(temp1); //arc temp2 -> temp1
		}
	    }
    }


  /*
   * Make interfere a wrote register with each live-out register
   */

  for(int i=0; i<cfg->node.size(); i++)
    {
      vector<int> write_reg = cfg->node[i]->get_write_register()->to_vector();
      //wrote register (just one, actually)

      vector<int> live_out_vec = cfg->live_out[i]->to_vector();
      //live-out registers

      for(int j=0; j<write_reg.size(); j++)
	for(int k=0; k<live_out_vec.size(); k++)
	  //for each couple (wrote_register,live-out-register)
	  {
	    int temp1 = write_reg[j];
	    int temp2 = live_out_vec[k];
	    if((temp1 != ARP) && (temp1 != SP) &&
	       (temp1 != SPILL0) && (temp1 != SPILL1) &&
	       (temp2 != ARP) && (temp2 != SP) &&
	       (temp2 != SPILL0) && (temp2 != SPILL1) &&
	       (temp1 != temp2))
	      {
		succ[temp1]->insert(temp2); //arc temp1 -> temp2
		succ[temp2]->insert(temp1); //arc temp2 -> temp1
	      }
	  }
    }
}


//Pretty-printers
void 
InterferenceGraph::print_dot(ostream& sout)
{
  sout << "digraph interfere {"<< endl;

  //Edges
  for(int i=0; i<succ.size(); i++)
    {
      vector<int> succ_vec = succ[i]->to_vector();
      for(int j=0; j<succ_vec.size(); j++)
	sout << "r" << i << " -> r" << succ_vec[j] << endl;
    }

  sout << "}";
}


void 
InterferenceGraph::show_dot()
{
  ofstream output_file;
  output_file.open("interferences.dot");
  print_dot(output_file);
  output_file.close();
  //Produce the .dot

  system("dot -Tps interferences.dot > interferences.ps");
  system("evince interferences.ps &");
  //Compile and display
}
